﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class BuildEngineer : EngineerModule
    {
        ThrustToWeight twr;

        bool drawStandard = true;       // Is this module set to draw the Standard Specifications display.
        bool drawStandardPrev;          // Stores the previous state of drawStandard when hiding displays.
        bool drawThrust = false;        // Is this module set to draw the Thrust to Weight Ratio display.
        bool drawThrustPrev;            // Stores the previous state of drawThrust when hiding displays.
        bool drawAllStages = false;     // Is this module showing All Stages, or just Important Stages.
        bool hideDisplays = false;      // Is the hide all displays button pressed.
        bool isEditorLocked = false;    // Is the editor locked due to mouse over window.

        // Module started.
        public override void OnStart(StartState state)
        {
            base.OnStart(state);

            // Check if started within the VAB.
            if (state == StartState.Editor)
            {
                // Associate editor events to callback methods.
                this.part.OnEditorAttach = OnEditorAttach;
                this.part.OnEditorDetach = OnEditorDetach;
                this.part.OnEditorDestroy = OnEditorDestroy;

                // Run the associated attach event method to start the GUI if ship was loaded.
                OnEditorAttach();

                Debug("BuildEngineer: OnStart");
            }
        }

        // Runs when this part is attached to the ship.
        public void OnEditorAttach()
        {
            // Check if this is the primary Engineer part on the ship.
            this.IsPrimary = CheckIsPrimary("BuildEngineer", EditorLogic.SortedShipList);

            // Start the module if it is the primary part and attached to the ship.
            if (IsPrimary && this.part.parent != null)
            {
                // Load the settings for window position.
                LoadSettings();

                // If no saved position was found, set default.
                if (winPos.x == 0 && winPos.y == 0)
                    winPos = new Rect(300, 70, 0, 0);

                // Create the ThrustToWeight object used for the TWR display.
                twr = new ThrustToWeight();

                // Update the ship object with all of the ship parts.
                ship.Update(EditorLogic.SortedShipList);

                // Add the DrawGUI callback method into the GUI rendering queue.
                RenderingManager.AddToPostDrawQueue(0, DrawGUI);

                Debug("BuildEngineer: OnEditorAttach");
            }
        }

        // Runs when this part is detached from the ship. (Can still be floating in VAB).
        public void OnEditorDetach()
        {
            // Check if this part is running as the primary Engineer part.
            if (IsPrimary)
            {
                // Remove the DrawGUI callback method from the GUI rendering queue.
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);

                Debug("BuildEngineer: OnEditorDetach");
            }
        }

        // Runs when this part is destroyed. (Deleted from VAB).
        public void OnEditorDestroy()
        {
            // Check if this part is running as the primary Engineer part.
            if (IsPrimary)
            {
                // Remove the DrawGUI callback method from the GUI rendering queue.
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);

                Debug("BuildEngineer: OnEditorDestroy");
            }
        }

        // Extends the basic DrawGUI method in EngineerModule.
        public override void DrawGUI()
        {
            base.DrawGUI();

            // If this part does not exist anymore.
            if (this.part == null)
            {
                // Remove from the GUI rendering queue and exit the method.
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);
                return;
            }

            // Display the window. (Including version info in window title).
            Rect winPosBefore = winPos;
            winPos = GUILayout.Window(3, winPos, MainWindow, "Kerbal Engineer Redux - Version " + Version, winStyle);

            // If the window has moved, save the new position.
            if (winPos != winPosBefore) SaveSettings();

            // Stop clickthrough of the window in the VAB.
            Vector2 mousePos = Input.mousePosition;
            mousePos.y = Screen.height - mousePos.y;
            if (winPos.Contains(mousePos) && !isEditorLocked && !EditorLogic.editorLocked)
            {
                EditorLogic.fetch.Lock();
                isEditorLocked = true;
            }
            else if (!winPos.Contains(mousePos) && isEditorLocked && EditorLogic.editorLocked)
            {
                EditorLogic.fetch.Unlock();
                isEditorLocked = false;
            }

            //Debug("BuildEngineer: DrawGUI");
        }

        // Extends the InitGUI method in the EngineerModule.
        protected override void InitGUI()
        {
            base.InitGUI();
            winStyle.fixedWidth = 700;  // Window width is 700px.
        }

        // Contains the drawing code for the main window.
        public void MainWindow(int windowID)
        {
            GUILayout.BeginHorizontal();
            
            // Set the width of the buttons to 250px.
            buttonStyle.fixedWidth = 175;
            buttonStyle.normal = HighLogic.Skin.button.normal;

            // Standard Specifications button.
            if (drawStandard)
            {
                buttonStyle.normal = buttonStyle.active;
            }
            if (GUILayout.Button("Standard Specifications", buttonStyle))
            {
                drawStandard = !drawStandard;
                winPos.height = 0;
            }

            buttonStyle.normal = HighLogic.Skin.button.normal;

            // Thrust to Weight Ratio button.
            if (drawThrust)
            {
                buttonStyle.normal = buttonStyle.active;
            }
            if (GUILayout.Button("Thrust to Weight", buttonStyle))
            {
                drawThrust = !drawThrust;
                winPos.height = 0;
            }

            //buttonStyle.fixedWidth = 150;
            buttonStyle.normal = HighLogic.Skin.button.normal;

            // Hide all displays button.
            if (hideDisplays && !drawStandard && !drawThrust)
            {
                buttonStyle.normal = buttonStyle.active;
            }
            else
            {
                hideDisplays = false;
            }
            if (GUILayout.Button("Hide Open Displays", buttonStyle))
            {
                if (!hideDisplays)
                {
                    if (drawStandard || drawThrust)
                    {
                        drawStandardPrev = drawStandard;
                        drawStandard = false;
                        drawThrustPrev = drawThrust;
                        drawThrust = false;
                    }
                    hideDisplays = true;
                }
                else
                {
                    if (!drawStandard && !drawThrust)
                    {
                        drawStandard = drawStandardPrev;
                        drawThrust = drawThrustPrev;
                    }
                    hideDisplays = false;
                }
                winPos.height = 0;
            }

            // Set the width of the buttons to be dynamic.
            buttonStyle.fixedWidth = 0;
            buttonStyle.normal = HighLogic.Skin.button.normal;

            // All Stages / Important Stages button.
            if (drawAllStages)
            {
                buttonStyle.normal = buttonStyle.active;
            }
            if (GUILayout.Button("All Stages", buttonStyle))
            {
                drawAllStages = !drawAllStages;
                winPos.height = 0;
            }
            GUILayout.EndHorizontal();

            // Draw the Standard Specifications display.
            if (drawStandard)
                DrawStandard();

            // Draw the Thrust to Weight Ratio display.
            if (drawThrust)
                DrawThrust();

            // Enable whole window to be draggable.
            GUI.DragWindow(new Rect(0, 0, winPos.width, winPos.height));
        }

        private void DrawStandard()
        {
            // Standard Specifications display heading.
            GUILayout.BeginHorizontal();
            GUILayout.Label("Standard Specifications", titleStyle);
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);

            // Stage Number.
            GUILayout.BeginVertical();
            GUILayout.Label("", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
            {
                if (ship.Stages[i].Important || drawAllStages)
                {
                    GUILayout.Label("S " + Convert.ToString(i), headStyle);
                }
            }
            GUILayout.EndVertical();

            // Stage Cost.
            GUILayout.BeginVertical();
            GUILayout.Label("Cost", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
            {
                if (ship.Stages[i].Important || drawAllStages)
                {
                    GUILayout.Label(ship.Stages[i].Cost.ToString("0") + " / " + ship.Stages[i].TotalCost.ToString("0"), dataStyle);
                }
            }
            GUILayout.EndVertical();

            // Stage Mass.
            GUILayout.BeginVertical();
            GUILayout.Label("Mass", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
            {
                if (ship.Stages[i].Important || drawAllStages)
                {
                    GUILayout.Label(ship.Stages[i].Mass.ToString("0.00") + " / " + ship.Stages[i].TotalMass.ToString("0.00"), dataStyle);
                }
            }
            GUILayout.EndVertical();

            // Stage Fuel.
            GUILayout.BeginVertical();
            GUILayout.Label("Fuel", headStyle);
            foreach (Stage stage in ship.Stages)
            {
                if (stage.Important || drawAllStages)
                {
                    if (stage.Fuel > 0)
                    {
                        GUILayout.Label(stage.Fuel.ToString("0"), dataStyle);
                    }
                    else
                    {
                        GUILayout.Label("-----", dataStyle);
                    }
                }
            }
            GUILayout.EndVertical();

            // Stage Isp.
            GUILayout.BeginVertical();
            GUILayout.Label("Isp(A/V)", headStyle);
            foreach (Stage stage in ship.Stages)
            {
                if (stage.Important || drawAllStages)
                {
                    if (stage.AtmIsp > 0 || stage.VacIsp > 0)
                    {
                    GUILayout.Label(stage.AtmIsp.ToString("0") + " / " + stage.VacIsp.ToString("0"), dataStyle);
                    }
                    else
                    {
                        GUILayout.Label("-----", dataStyle);
                    }
                }
            }
            GUILayout.EndVertical();

            // Stage Thrust.
            GUILayout.BeginVertical();
            GUILayout.Label("Thrust", headStyle);
            foreach (Stage stage in ship.Stages)
            {
                if (stage.Important || drawAllStages)
                {
                    if (stage.Thrust > 0)
                    {
                        GUILayout.Label(stage.Thrust.ToString("0"), dataStyle);
                    }
                    else
                    {
                        GUILayout.Label("-----", dataStyle);
                    }
                }
            }
            GUILayout.EndVertical();

            // Stage Delta-V.
            GUILayout.BeginVertical();
            GUILayout.Label("Delta-V", headStyle);
            float totalDeltaV = ship.Stages[ship.Stages.Count - 1].TotalDeltaV;
            foreach (Stage stage in ship.Stages)
            {
                if (stage.Important || drawAllStages)
                {
                    if (stage.DeltaV > 0)
                    {
                        GUILayout.Label(stage.DeltaV.ToString("0") + " / " + totalDeltaV.ToString("0") + " m/sec", dataStyle);
                        totalDeltaV -= stage.DeltaV;
                    }
                    else
                    {
                        GUILayout.Label("-----", dataStyle);
                    }
                }
            }
            GUILayout.EndVertical();

            GUILayout.EndHorizontal();
        }

        private void DrawThrust()
        {
            // Thrust to Weight Ratios display heading.
            GUILayout.BeginHorizontal();
            GUILayout.Label("Thrust to Weight Ratios", titleStyle);
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);

            // Stage Number.
            GUILayout.BeginVertical();
            GUILayout.Label("", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
            {
                if (ship.Stages[i].Important || drawAllStages)
                {
                    GUILayout.Label("S " + Convert.ToString(i), headStyle);
                }
            }
            GUILayout.EndVertical();

            // Loop through all of the names in the GravityList
            foreach (string influenceName in twr.GravityList.Keys.ToArray<string>())
            {
                GUILayout.BeginVertical();
                GUILayout.Label(influenceName, headStyle);

                // Loop through all of the stages.
                foreach (Stage stage in ship.Stages)
                {
                    // Check if stage is considered important or if all stages are to be drawn.
                    if (stage.Important || drawAllStages)
                    {
                        // Calculate the thrust to weight ratio of the stage.
                        float thrustToWeight = stage.Thrust / (stage.TotalMass * twr.GravityList[influenceName]);

                        if (thrustToWeight > 0)
                        {
                            // Display thrust to weight ratio.
                            GUILayout.Label(thrustToWeight.ToString("0.00"), dataStyle);
                        }
                        else
                        {
                            GUILayout.Label("-----", dataStyle);
                        }                        
                    }
                }
                GUILayout.EndVertical();
            }

            GUILayout.EndHorizontal();
        }

        // Runs whenever a new part is added or removed from the ship.
        public override void OnSave(ConfigNode node)
        {
            base.OnSave(node);

            // Check if in editor and this part is the primary Engineer.
            if (state == StartState.Editor && IsPrimary)
            {
                // Update the ship with a complete set of new parts.
                ship.Update(EditorLogic.SortedShipList);

                // Reset window height to 0 so it can rescale.
                winPos.height = 0;

                Debug("BuildEngineer: OnSave");
            }
        }

        public override void OnLoad(ConfigNode node)
        {
            base.OnLoad(node);

            LoadSettings();
        }

        private void SaveSettings()
        {
            settings.SetValue("BuildEngineer::winPos.x", winPos.x.ToString());
            settings.SetValue("BuildEngineer::winPos.y", winPos.y.ToString());
            settings.Save();
        }

        private void LoadSettings()
        {
            settings.Load();
            winPos.x = Convert.ToSingle(settings.GetValue("BuildEngineer::winPos.x"));
            winPos.y = Convert.ToSingle(settings.GetValue("BuildEngineer::winPos.y"));
        }
    }
}
