﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class EngineerModule : PartModule
    {
        // Set the version of this plugin here.
        public const string PluginName = "Kerbal Engineer Redux";
        public const string Version = "0.4.4";

        public enum SIUnitType { Speed, Distance };     // Enumeration used when formatting numbers into SI units for display.
        public Settings settings = new Settings();      // Used for saving/loading window position data from the file system.
        public StartState state = StartState.None;      // Start state used to check if in Flight or Editor mode.
        public Ship ship;                               // Ship object used in both Build & Flight Engineers.
        public Rect winPos;                             // Object to hold the window position.
        public GUIStyle winStyle, areaStyle, titleStyle, headStyle, dataStyle, buttonStyle, scrollStyle;    // GUI styles.
        public bool hasInitGUI = false;                 // Has the part initialised the GUI styles.

        // Property used to determine if this part is the primary Engineer on the ship.
        public bool IsPrimary { get; protected set; }

        // Get a specific stage from the ship object.
        public Stage GetStage(int number)
        {
            // Check if stage available.
            if (number < ship.Stages.Count)
            {
                return ship.Stages[number];
            }
            else
            {
                // If stage is out of bounds, return the last stage which is in bounds.
                return ship.Stages[ship.Stages.Count - 1];
            }
        }

        // Module started.
        public override void OnStart(StartState state)
        {
            base.OnStart(state);
            this.state = state;

            // Instantiate an object of Ship which can be used by the Build & Flight Engineer.
            ship = new Ship(this);

            Debug("EngineerModule: OnStart(" + state + ")");
        }

        // This is here purely for debug purposes (not used).
        public override void OnSave(ConfigNode node)
        {
            foreach (ConfigNode.Value value in node.values)
            {
                Debug("EngineerModule (Node): " + value.name + " = " + value.value);
            }
            Debug("EngineerModule: OnSave");
        }

        // This is here purely for debug purposes (not used).
        public override void OnLoad(ConfigNode node)
        {
            foreach (ConfigNode.Value value in node.values)
            {
                Debug("EngineerModule (Node): " + value.name + " = " + value.value);
            }
            Debug("EngineerModule: OnLoad");
        }

        // Check whether this part is the primary part on the ship.
        protected bool CheckIsPrimary(string className, List<Part> partList)
        {
            // Check if this part is attached to anything.
            if (this.part.parent == null)
                return false;

            // Loop through all parts on the ship.
            foreach (Part part in partList)
            {
                // Check if it is not this part and is an EngineerModule part.
                if (part != this.part && part.Modules.Contains(className))
                {
                    // Convert part to type EngineerModule to access it's properties.
                    EngineerModule partMod = (part.Modules[className] as EngineerModule);

                    // Check if the part is primary already.
                    if (partMod.IsPrimary)
                    {
                        // If the part is primary, return that this part cannot be primary.
                        Debug("EngineerModule: CheckIsPrimary(" + className + ") = false");
                        return false;
                    }
                }
            }

            // If no other parts found to be primary, return that this part can be primary.
            Debug("EngineerModule: CheckIsPrimary(" + className + ") = true");
            return true;
        }

        // DrawGUI method that can be extended via inheriting classes.
        virtual public void DrawGUI()
        {
            // Set global GUI skin to HighLogic.
            GUI.skin = HighLogic.Skin;

            // If the GUI styles have not been initialised, initialise them.
            if (!hasInitGUI)
                InitGUI();
        }

        // Method for initialising all of the GUI styles.
        virtual protected void InitGUI()
        {
            winStyle = new GUIStyle(HighLogic.Skin.window);
            winStyle.stretchHeight = true;

            areaStyle = new GUIStyle(HighLogic.Skin.box);
            areaStyle.hover = areaStyle.active = areaStyle.normal;

            titleStyle = new GUIStyle(HighLogic.Skin.label);
            titleStyle.normal.textColor = Color.yellow;
            titleStyle.fontStyle = FontStyle.Bold;

            headStyle = new GUIStyle(HighLogic.Skin.label);
            headStyle.stretchWidth = true;
            headStyle.normal.textColor = Color.white;
            headStyle.fontStyle = FontStyle.Bold;
            headStyle.wordWrap = false;

            dataStyle = new GUIStyle(HighLogic.Skin.label);
            dataStyle.stretchWidth = true;
            dataStyle.wordWrap = false;

            buttonStyle = new GUIStyle(HighLogic.Skin.button);
            buttonStyle.stretchWidth = true;

            scrollStyle = new GUIStyle(HighLogic.Skin.scrollView);
            scrollStyle.padding = areaStyle.padding;

            // Set that this module has initialised the GUI styles.
            hasInitGUI = true;

            Debug("EngineerModule: InitGUI");
        }
        

        // Function for formatting a number ready to display in selected type of SI unit.
        public string FormatSI(double number, SIUnitType type)
        {
            // Assign memory for storing the notations.
            string[] notation = { "" };

            // Select the SIUnitType used and populate the notation array.
            switch (type)
            {
                case SIUnitType.Distance:
                    notation = new string[] { "m", "km", "Mm", "Gm", "Tm", "Pm", "Em", "Ym", "Zm" };
                    break;
                case SIUnitType.Speed:
                    notation = new string[] { "mm/s", "m/s", "km/s", "Mm/s", "Gm/s", "Tm/s", "Pm/s", "Em/s", "Ym/s", "Zm/s" };
                    number *= 1000;
                    break;
            }

            int notationIndex = 0;  // Index that is used to select the notation to display.

            // Loop through the notations until the smallest usable one is found.
            for (notationIndex = 0; notationIndex < notation.Length; notationIndex++)
            {
                if (number > 1000 || number < -1000) { number /= 1000; } else { break; }
            }

            // Return a string of the concatinated number and selected notation.
            return number.ToString("0.000") + " " + notation[notationIndex];
        }

        // Conditional method where it and all calls to it are only compiled when debug flag is present.
        [Conditional("DEBUG")]
        public void Debug(string message)
        {
            print(message);
        }
    }
}
