﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class FlightEngineer : EngineerModule
    {
        public Vessel activeVessel;     // Your current vessel.
        Rendevous rendevous = new Rendevous();     // Used for the rendezvous display.
        bool windowDraggable = true;    // Option for making the window draggable or not.
        bool engineerStarted = true;    // Has the Engineer been started. (Required for ship swapping).
        bool showOrbital = false;       // Toggle for showing Orbital information.
        bool showSurface = false;       // Toggle for showing Surface information.
        bool showVessel = false;        // Toggle for showing Vessel information.
        bool showRendevous = false;     // Toggle for show the Rendezvous display.
        double maxGForce = 0.0f;        // Variable used to stop maximum g-force achieved.

        // Module started.
        public override void OnStart(StartState state)
        {
            base.OnStart(state);

            // Try and start the flight engineer.
            StartEngineer();

            Debug("FlightEngineer: OnStart(" + state + ")");
        }

        // Frame update.
        public override void OnUpdate()
        {
            base.OnUpdate();

            // Check if engineer has been started.  If not, try and start it.
            if (!engineerStarted)
                StartEngineer();

            //Debug("FlightEngineer: OnUpdate");
        }

        // Method used to start the Flight Engineer.
        public void StartEngineer()
        {
            // Only start if not in the VAB and the vessel this part is attached to is the current active vessel.
            if (state != StartState.Editor && this.vessel.rootPart == FlightGlobals.ActiveVessel.rootPart)
            {
                // Store this as the current active vessel for future use.
                activeVessel = this.vessel;

                // Check if this is the primary Engineer part on the ship.
                IsPrimary = CheckIsPrimary("FlightEngineer", activeVessel.parts);

                // Start the Flight Engineer if it is the primary Engineer on the ship.
                if (IsPrimary)
                {
                    // Load the settings for the window position.
                    LoadSettings();

                    // If no saved position was found, set default.
                    if (winPos.x == 0 && winPos.y == 0)
                        winPos = new Rect(UnityEngine.Screen.width - 275, 0, 0, 0);

                    // Add the DrawGUI callback method into the GUI rendering queue.
                    RenderingManager.AddToPostDrawQueue(0, DrawGUI);

                    // Instantiate the Rendezvous object with this module.
                    rendevous.FlightEngineer = this;

                    // Force this part to activate.
                    this.part.force_activate();

                    // Set that the engineer has now started and exit method.
                    engineerStarted = true;
                    return;
                }
            }

            // Set that the engineer has not been started.
            engineerStarted = false;
        }

        // Extends the DrawGUI method.
        public override void DrawGUI()
        {
            base.DrawGUI();

            // Check if this part does not exist, is not attached or is not part of the active vessel anymore.
            if (this.part == null || this.part.isAttached == false || activeVessel != FlightGlobals.ActiveVessel)
            {
                // Remove the DrawGUI callback method from the GUI rendering queue.
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);
                engineerStarted = false;

                // Exit method.
                return;
            }

            // Update the ship with all the parts of this active vessel.
            ship.Update(activeVessel.parts);

            // Display the window.
            Rect winPosBefore = winPos;
            winPos = GUILayout.Window(3, winPos, MainWindow, "Flight Engineer", winStyle);

            // If the window has moved, save the new position.
            if (winPos != winPosBefore) SaveSettings();
        }

        // Extends the InitGUI method.
        protected override void InitGUI()
        {
            base.InitGUI();
            winStyle.fixedWidth = 275;
            dataStyle.alignment = TextAnchor.UpperRight;
        }

        // Draws the main window.
        public void MainWindow(int windowId)
        {
            // Draw the option buttons at the top.
            DrawOptions();

            // Draw the orbital display if the option is set.
            if (showOrbital)
                DrawOrbital();

            // Draw the surface display if the option is set.
            if (showSurface)
                DrawSurface();

            // Draw the vessel display if the option is set.
            if (showVessel)
                DrawVessel();

            // Draw the rendezvous display if the option is set.
            if (showRendevous)
                DrawRendevous();

            // Allow the window to be draggable if it is set to be.
            if (windowDraggable) GUI.DragWindow(new Rect(0, 0, winPos.width, winPos.height));
        }

        // Draws the top option buttons.
        private void DrawOptions()
        {
            GUILayout.BeginHorizontal();
            GUILayout.BeginVertical();

            // Check if the orbital display is active.
            if (showOrbital)
            {
                // If it is active then show the button as pressed.
                buttonStyle.normal = HighLogic.Skin.button.active;
            }
            else
            {
                // If it is not active then dhow the button as up.
                buttonStyle.normal = HighLogic.Skin.button.normal;
            }

            // Draw the button.
            if (GUILayout.Button("ORB", buttonStyle))
            {
                showOrbital = !showOrbital;
                winPos.height = 0;
            }

            GUILayout.EndVertical();
            GUILayout.BeginVertical();

            // Check if the surface display is active.
            if (showSurface)
            {
                // If it is active then show the button as pressed.
                buttonStyle.normal = HighLogic.Skin.button.active;
            }
            else
            {
                // If it is not active then show the button as up.
                buttonStyle.normal = HighLogic.Skin.button.normal;
            }

            // Draw the button.
            if (GUILayout.Button("SUR", buttonStyle))
            {
                showSurface = !showSurface;
                winPos.height = 0;
            }

            GUILayout.EndVertical();
            GUILayout.BeginVertical();

            // Check if the vessel display is active.
            if (showVessel)
            {
                // If it is active then show the button as pressed.
                buttonStyle.normal = HighLogic.Skin.button.active;
            }
            else
            {
                // If it is not active then dhow the button as up.
                buttonStyle.normal = HighLogic.Skin.button.normal;
            }

            // Draw the button.
            if (GUILayout.Button("VES", buttonStyle))
            {
                showVessel = !showVessel;
                winPos.height = 0;
            }

            GUILayout.EndVertical();
            GUILayout.BeginVertical();

            // Check if the vessel display is active.
            if (showRendevous)
            {
                // If it is active then show the button as pressed.
                buttonStyle.normal = HighLogic.Skin.button.active;
            }
            else
            {
                // If it is not active then dhow the button as up.
                buttonStyle.normal = HighLogic.Skin.button.normal;
            }

            // Draw the button.
            if (GUILayout.Button("RDV", buttonStyle))
            {
                showRendevous = !showRendevous;
                winPos.height = 0;
            }

            // Reset the button style to normal for any future buttons which may want to use it.
            buttonStyle.normal = HighLogic.Skin.button.normal;

            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        // Draws the orbital display.
        private void DrawOrbital()
        {
            
            // Display title.
            GUILayout.Label("Orbital Display", titleStyle);
            
            // Heading column.
            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Apoapsis Height", headStyle);
            GUILayout.Label("Periapsis Height", headStyle);
            GUILayout.Label("Time to Apoapsis", headStyle);
            GUILayout.Label("Time to Periapsis", headStyle);
            GUILayout.Label("Inclination", headStyle);
            GUILayout.Label("Eccentricity", headStyle);
            GUILayout.Label("Orbital Period", headStyle);
            GUILayout.Label("Semi-major Axis", headStyle);
            GUILayout.EndVertical();

            // Data column.
            GUILayout.BeginVertical();
            GUILayout.Label(FormatSI(activeVessel.orbit.ApA, SIUnitType.Distance), dataStyle);
            GUILayout.Label(FormatSI(activeVessel.orbit.PeA, SIUnitType.Distance), dataStyle);
            GUILayout.Label(activeVessel.orbit.timeToAp.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(activeVessel.orbit.timeToPe.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(activeVessel.orbit.inclination.ToString("0.000") + "°", dataStyle);
            GUILayout.Label(activeVessel.orbit.eccentricity.ToString("0.000000"), dataStyle);
            GUILayout.Label(activeVessel.orbit.period.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(FormatSI(activeVessel.orbit.semiMajorAxis, SIUnitType.Distance), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        // Draws the surface display.
        private void DrawSurface()
        {
            // Set the maximum g-force experienced.
            if (activeVessel.geeForce > maxGForce) maxGForce = activeVessel.geeForce;

            // Display title.
            GUILayout.Label("Surface Display", titleStyle);

            // Heading column.
            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Altitude (Sea Level)", headStyle);
            GUILayout.Label("Altitude (Terrain)", headStyle);
            GUILayout.Label("Vertical Speed", headStyle);
            GUILayout.Label("Horizontal Speed", headStyle);
            GUILayout.Label("Longitude", headStyle);
            GUILayout.Label("Latitude", headStyle);
            GUILayout.Label("G-Force", headStyle);
            GUILayout.Label("Atmospheric Density", headStyle);
            GUILayout.EndVertical();

            // Data column.
            GUILayout.BeginVertical();
            GUILayout.Label(FormatSI(activeVessel.mainBody.GetAltitude(activeVessel.CoM), SIUnitType.Distance), dataStyle);
            GUILayout.Label(FormatSI(activeVessel.mainBody.GetAltitude(activeVessel.CoM) - activeVessel.terrainAltitude, SIUnitType.Distance), dataStyle);
            GUILayout.Label(FormatSI(activeVessel.verticalSpeed, SIUnitType.Speed), dataStyle);
            GUILayout.Label(FormatSI(activeVessel.horizontalSrfSpeed, SIUnitType.Speed), dataStyle);
            GUILayout.Label(activeVessel.longitude.ToString("0.000000") + "°", dataStyle);
            GUILayout.Label(activeVessel.latitude.ToString("0.000000") + "°", dataStyle);
            GUILayout.Label(activeVessel.geeForce.ToString("0.000") + " / " + maxGForce.ToString("0.000"), dataStyle);
            GUILayout.Label(activeVessel.atmDensity.ToString("0.000000"), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        } 

        // Draws the vessel display.
        private void DrawVessel()
        {
            // Display title.
            GUILayout.Label("Vessel Display", titleStyle);

            // Heading column.
            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Delta-V (Stage)", headStyle);
            GUILayout.Label("Delta-V (Total)", headStyle);
            GUILayout.Label("Fuel", headStyle);
            GUILayout.Label("Mass", headStyle);
            GUILayout.Label("Thrust", headStyle);
            GUILayout.Label("TWR", headStyle);
            GUILayout.EndVertical();

            // Data column.
            GUILayout.BeginVertical();
            GUILayout.Label(GetStage(activeVessel.currentStage).DeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).TotalDeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).Fuel.ToString("0.00") + " / " + GetStage(activeVessel.currentStage).TotalFuel.ToString("0.00"), dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).Mass.ToString("0.00") + " / " + GetStage(activeVessel.currentStage).TotalMass.ToString("0.00"), dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).Thrust.ToString("0") + " kN", dataStyle);
            GUILayout.Label((GetStage(activeVessel.currentStage).Thrust / (GetStage(activeVessel.currentStage).TotalMass * (activeVessel.mainBody.gravParameter / Math.Pow(activeVessel.mainBody.Radius, 2)))).ToString("0.00"), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        // Draws the rendezvous display.
        private void DrawRendevous()
        {
            rendevous.Draw();
        }

        private void SaveSettings()
        {
            settings.SetValue("FlightEngineer::winPos.x", winPos.x.ToString());
            settings.SetValue("FlightEngineer::winPos.y", winPos.y.ToString());
            settings.Save();
        }

        private void LoadSettings()
        {
            settings.Load();
            winPos.x = Convert.ToSingle(settings.GetValue("FlightEngineer::winPos.x"));
            winPos.y = Convert.ToSingle(settings.GetValue("FlightEngineer::winPos.y"));
        }
    }
}
